-- MacroScript NS_PhoenixPresets category:"Nik Scripts" toolTip:"Phoenix Presets" 
(

	/*
	##########################################################################################################################
	VARIABLES
	##########################################################################################################################
	*/
	scriptVersion = "0.29"
	global nsFloaterPhPresets
	global nsPhPresetsIni
	if nsPhPresetsIni == undefined then nsPhPresetsIni = "Pick ini file"
	rolPhPresets	
	
	/*
	##########################################################################################################################
	ROLLOUT
	##########################################################################################################################
	*/
	Rollout rolPhPresets "Settings" (
		edittext edtIni "Ini File: " text:nsPhPresetsIni fieldWidth:(rolPhPresets.width-90) across:2
		button btnPickIni "..." height:16 width:16 align:#right
		checkbox chkSaveGpuPreview "Save GPU Preview" checked:false

		checkbox chkSimulate "Simulate" checked:true align:#right
		button btnStart "START" width:60 height:30 align:#right

		radiobuttons radRender "Render:" labels:#("No render","After each simulations","After all simulation") columns:1 align:#left offset:[0,-55]
		checkbox chkFramestamp "Framestamp" checked:true enabled:(radRender.state>1) align:#left
		edittext edtStatus "Job status: " text:"standby" readOnly:true 
		
		/*
		##########################################################################################################################
		FUNCTIONS
		##########################################################################################################################
		*/
		local phObj
		local phCachePath=""
		local phPresetString=""
		
		-- GRAB GPU VIEWPORT WITH ASPECT RATIO
		fn getViewportSafeframeDib show:off = (
			local x = y = 0, v
			local s = getViewSize()
			local k = getRendImageAspect()

			if k < (s.x/s.y) then (
				v = [s.y*k,s.y]
				x = (s.x-v.x)/2 + 1
			)
			else (
				v = [s.x, s.x/k]
				y = (s.y-v.y)/2 + 1
			)
			bmp = bitmap v.x v.y
			pasteBitmap (gw.getViewportDib()) bmp (box2 x y v.x v.y) [0,0]
			if show do display bmp
			bmp
		)
		-- REPLACE PHOENIX PATH TAGS	
		fn PhGetCachePath = (
			str = phObj.simoutput
			str = substitutestring str "$(scene_dir)" "$(dir)\$(scene)_Phoenix2_frames\ND$(handle)_$(nodename)_####.aur"
			str = substitutestring str "$(implicit)" "$(dir)\$(scene)_Phoenix2_frames\ND$(handle)_$(nodename)_####.aur"
			str = substitutestring str "$(dir)" maxfilepath
			str = substitutestring str "$(scene)" maxfilename
			str = substitutestring str "$(handle)" (phObj.inode.handle as string)
			str = substitutestring str "$(nodename)" (phObj.name)
			str
		)
		-- RENDER PHOENIX SEQUENCE
		fn PhStartRender frStart frEnd filePath = (
			vr = renderers.current
			-- vray settings
			vr.imageSampler_type = 1 
			vr.system_frameStamp_on = chkFramestamp.checked
			vr.system_frameStamp_string = phPresetString
			vr.output_saveRawFile = true
			makedir (getFilenamePath filePath)
			renderSceneDialog.close()
			rendTimeType = 3
			rendStart = frStart
			rendEnd = frEnd
			rendSaveFile = true
			rendOutputFilename = filePath + "_jpgRender.jpg"			
			vr.output_rawFileName = filePath + "_rawRender.exr"
			max quick render
		)	
		-- LOAD PRESET SETTINGS
		fn PhLoadPreset presetName = (
			propsArr = getINISetting nsPhPresetsIni presetName
			phPresetString=""
			-- set all properties in preset
			for prop in propsArr where hasproperty phObj prop do (
				val = getINISetting nsPhPresetsIni presetName prop
				valClass = classof (getProperty phObj prop)
				-- convert preset tag to preset name in cache path
				if prop == "simoutput" then val = substitutestring val "$(preset)" presetName
					-- append property string for framestamp
					else append phPresetString (prop + ": " + val as string + " | ")
				setProperty phObj prop (val as valClass)				
			)
			-- get sim cache path
			phCachePath = getFilenamePath (PhGetCachePath())
		)
		-- FORMAT TIME INTERVAL TO STRING
		fn TimeFormatting timeMs = (
			timeSec = timeMs / 1000
			timeString = ((dotnetclass "TimeSpan").FromSeconds (timeSec)).ToString()
		)
		-- START PHOENIX SIMULATION AND RENDER
		fn PhPresetsStart renderType:0 gpuPreview:false = (
			timeJobArr=#()
			-- save sim cache path
			oldCachePath = phObj.simoutput
			presetsArr = getINISetting nsPhPresetsIni
			-- simulate presets
			for presetName in presetsArr do (
				-- load preset settings and return normalized sim cache path
				PhLoadPreset presetName
				-- start simulation
				if chkSimulate.checked then (
					edtStatus.text = ("Sim " + presetName + " started...")
					-- start preset timer
					presetTimeStart = timeStamp()
					A_StartSim phObj
					A_Wait phObj
					-- end preset timer and get time
					presetTimeEnd = timeStamp()
					presetTimeMs = (presetTimeEnd - presetTimeStart)
					presetTime = TimeFormatting (presetTimeEnd - presetTimeStart)
					-- save time to ini
					SetIniSetting nsPhPresetsIni presetName "preset_sim_time" presetTime
					append timeJobArr presetTimeMs
				)
				-- save gpu preview
				if gpuPreview then (
					hideByCategory.helpers = true
					dirGpuPreview = phCachePath + "\\GpuPreview\\" 
					makedir dirGpuPreview
					phObj.gpu_viewport = on
					phObj.showgrid = off
					for t = phObj.startframe to phObj.stopframe do (
						sliderTime = t
						bm = getViewportSafeframeDib()
						bm.filename = dirGpuPreview + presetName + "_gpuPreview.jpg"
						save bm frame:t quiet:true
					)
					phObj.showgrid = on
					hideByCategory.helpers = false
				)
				-- render after each simulation
				if renderType == 2 then (
					dirRender = phCachePath + "\\Render\\"
					filePath = (dirRender + presetName)
					PhStartRender phObj.startframe phObj.stopframe filePath
				)
				-- restore sim cache path
				phObj.simoutput = oldCachePath
			)
			-- render all presets after simulations
			if renderType == 3 then (
				for presetName in presetsArr do (
					PhLoadPreset presetName 
					dirRender = phCachePath + "\\Render\\"
					filePath = (dirRender + presetName)
					PhStartRender phObj.startframe phObj.stopframe filePath
					-- restore sim cache path
					phObj.simoutput = oldCachePath
				)
			)
			-- get job time
			timeJobAll=0
			for t in timeJobArr do timeJobAll = timeJobAll + t
			timeJob = TimeFormatting timeJobAll
			edtStatus.text = ("Job ended in: " + timeJob)
			btnStart.enabled = true
		)
		
		/*
		##########################################################################################################################
		EVENTS
		##########################################################################################################################
		*/
		on btnPickIni pressed do (
			pickFile = getOpenFileName caption:"Pick presets ini file"
			if pickFile != undefined and getFilenameType pickFile == ".ini" then (
				edtIni.text = pickFile
				nsPhPresetsIni = pickFile
			)
			else (
				edtIni.text = "Pick ini file"
				nsPhPresetsIni = edtIni.text
			)
		)
		on edtIni changed txt do (
			nsPhPresetsIni = txt
		)
		on radRender changed state do chkFramestamp.enabled = state > 1
		on btnStart pressed do (
			phObj = (for obj in selection where classof obj == FireSmokeSim collect obj)[1]
			if phObj != undefined then (
				if getFilenameType nsPhPresetsIni == ".ini" then (
					btnStart.enabled = false
					PhPresetsStart renderType:radRender.state gpuPreview:chkSaveGpuPreview.checked
				)
				else messagebox "Ini file is invalid"
			)
			else messagebox "Select Phoenix object"
		)
	)
	
	-- CLOSE AND CREATE FLOATER
	try (CloseRolloutFloater nsFloaterPhPresets) catch()
	nsFloaterPhPresets = NewRolloutFloater ("Phoenix Presets v." + scriptVersion) 400 190 200 200
	AddRollout rolPhPresets nsFloaterPhPresets
)
